Struct ExponentialBackoff

pub struct ExponentialBackoff<F, R = HasherRng> { /* private fields */ }
A jittered exponential backoff strategy.

The backoff duration will increase exponentially for every subsequent backoff, up to a maximum duration. A small amount of random jitter is added to each backoff duration, in order to avoid retry spikes.

Implementations

Source

impl<F, R> ExponentialBackoff<F, R>
where R: Rng + Clone, F: Fn() -> R + Clone,

pub fn new( min: Duration, max: Duration, jitter: f64, rng_creator: F, ) -> Result<Self, InvalidBackoff>

Create a new ExponentialBackoff.

Error

Returns a config validation error if:

  • min > max
  • max == 0
  • jitter < 0.0
  • jitter > 100.0
  • jitter is not finite