Crate rama_utils

Crate rama_utils 

utilities crate for rama

rama-utils contains utilities used by rama, not really being part of one of the other crates, or used by plenty of other crates.

Rama

Crate used by the end-user rama crate and rama crate authors alike.

Learn more about rama:

Modules

backoffstd
This module contains generic backoff utilities to be used with the retry and limit layers.
byte_set
const builders for [bool; 256] byte-set lookup tables.
bytes
byte-slice utilities
collections
Collections such as NonEmptyVec and NonEmptySmallVec provided by rama, mostly for internal usage, but available for others to use as well.
fsstd
Filesystem helpers shared by Rama crates.
hex
Small ASCII-hex utilities for single-byte parse/format.
include_dirstd
An extension to the include_str!() and include_bytes!() macro for embedding an entire directory tree into your binary.
info
Info constants about the rama crate.
latency
latency utilities and common types
octets
reactivestd
A lock-free-read value paired with a race-free change signal.
rng
PRNG utilities for middleware.
str
string utilities
thirdpartystd
Thirdparty utilities.
timestd
Time utilities providing a high-performance, cached wall clock.