utilities crate for rama
rama-utils contains utilities used by rama,
not really being part of one of the other crates, or used
by plenty of other crates.
Rama
Crate used by the end-user rama crate and rama crate authors alike.
Learn more about rama:
- Github: https://github.com/plabayo/rama
- Book: https://ramaproxy.org/book/
Modules
- backoff
std - This module contains generic backoff utilities to be used with the retry and limit layers.
- byte_
set constbuilders for[bool; 256]byte-set lookup tables.- bytes
- byte-slice utilities
- collections
- Collections such as
NonEmptyVecandNonEmptySmallVecprovided by rama, mostly for internal usage, but available for others to use as well. - fs
std - Filesystem helpers shared by Rama crates.
- hex
- Small ASCII-hex utilities for single-byte parse/format.
- include_
dir std - An extension to the
include_str!()andinclude_bytes!()macro for embedding an entire directory tree into your binary. - info
- Info constants about the rama crate.
- latency
- latency utilities and common types
- octets
- reactive
std - A lock-free-read value paired with a race-free change signal.
- rng
- PRNG utilities for middleware.
- str
- string utilities
- thirdparty
std - Thirdparty utilities.
- time
std - Time utilities providing a high-performance, cached wall clock.